package es.verdnatura.presentation.view.commom

import android.content.Context
import android.graphics.Color
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.LinearLayout
import androidx.appcompat.widget.SearchView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.R

class SearchableRecyclerView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : LinearLayout(context, attrs, defStyleAttr) {

    val searchView: SearchView
    private val recyclerView: RecyclerView
    lateinit var adapter: SearchableAdapter
    private var allItems: MutableList<NameWithId> = mutableListOf()

    init {
        LayoutInflater.from(context).inflate(R.layout.component_searchable_dialog, this, true)
        searchView = findViewById(R.id.search_view)
        recyclerView = findViewById(R.id.recycler_viewer)
        recyclerView.layoutManager = LinearLayoutManager(context)
        recyclerView.setBackgroundColor(Color.WHITE)
        searchView.setBackgroundColor(Color.WHITE)
        searchView.setOnCloseListener {
            visibility = View.GONE
            true
        }

        searchView.setOnQueryTextListener(object : SearchView.OnQueryTextListener {
            override fun onQueryTextSubmit(query: String?): Boolean {
                return false
            }

            override fun onQueryTextChange(newText: String?): Boolean {
                if (::adapter.isInitialized) adapter.filter(newText ?: "")
                return true
            }
        })
    }

    fun setAdapter(adapter: SearchableAdapter, items: MutableList<NameWithId>) {
        this.adapter = adapter
        this.allItems = items
        recyclerView.adapter = adapter
        adapter.notifyDataSetChanged()
    }

    fun setOnCloseListener(listener: () -> Boolean) {
        searchView.setOnCloseListener { listener() }
    }

    fun setVisibility() {
        searchView.visibility = View.VISIBLE
        recyclerView.visibility = View.VISIBLE
    }

    fun setSearchHint(searchHint: String): SearchableRecyclerView {
        searchView.queryHint = searchHint
        searchView.setIconifiedByDefault(true)
        searchView.isIconified = false
        searchView.post {
            searchView.requestFocus()
        }

        return this
    }

    fun setSearchHintWithoutFocus(searchHint: String): SearchableRecyclerView {
        searchView.queryHint = searchHint
        return this
    }
}
