package es.verdnatura.presentation.view.feature.articulo.fragment

import android.content.Context
import android.graphics.drawable.Drawable
import android.view.View
import android.widget.ImageView
import androidx.core.content.ContextCompat
import androidx.core.content.res.ResourcesCompat
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentItemproposalBinding
import es.verdnatura.domain.ConstAndValues.WAREHOUSEFK
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnItemProposalClickListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.common.ToolBarAdapterTooltip
import es.verdnatura.presentation.view.feature.articulo.adapter.ItemProposalAdapter
import es.verdnatura.presentation.view.feature.articulo.model.ItemProposal
import java.text.SimpleDateFormat
import java.util.Calendar
import java.util.Locale

class ItemProposalFragment(
    var itemFk: Int
) : BaseFragment<FragmentItemproposalBinding, ItemCardViewModel>(
    ItemCardViewModel::class
) {
    private var adapter: ItemProposalAdapter? = null
    private var listProposal: ArrayList<ItemProposal> = ArrayList()
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var layoutManager: LinearLayoutManager? = null
    private var filter = true

    companion object {
        fun newInstance(entryPoint: Int) = ItemProposalFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_itemproposal

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    private fun currentDay(): String {
        val calendar = Calendar.getInstance()
        val format = SimpleDateFormat(getString(R.string.dateFormat), Locale.getDefault())
        return format.format(calendar.time)
    }

    override fun init() {
        ma.hideBottomNavigation(View.GONE)
        "${getString(R.string.ArticleSimilar)}:$itemFk".also { it ->
            it.also {
                binding.mainToolbar.toolbarTitle.text = it
            }
        }
        setToolBar()
        setEvents()
        viewModel.itemGetSimilar(
            itemFk,
            mobileApplication.dataStoreApp.readDataStoreKey(WAREHOUSEFK),
            shipped = currentDay(),
            isShowedByType = filter,
        )
        super.init()
    }

    private fun setToolBar() {
        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconFilter = ImageView(context)
        iconFilter.setImageResource(R.drawable.filter_outline)

        iconFilter.tooltipText = getTooltip(R.drawable.filter_outline)
        ResourcesCompat.getDrawable(resources, R.drawable.filter_outline, resources.newTheme())

        listIcons.add(iconFilter)
        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {
                override fun onOptionsItemSelected(item: Drawable) {
                    when (item) {
                        iconFilter.drawable -> {
                            iconFilter.drawable.setTint(
                                ContextCompat.getColor(
                                    context!!,
                                    if (filter) R.color.verdnatura_white else R.color.verdnatura_pumpkin_orange
                                )
                            )
                            filter = !filter

                            viewModel.itemGetSimilar(
                                itemFk,
                                mobileApplication.dataStoreApp.readDataStoreKey(WAREHOUSEFK),
                                isShowedByType = filter,
                                shipped = currentDay(),
                            )
                        }
                    }
                }

            })
        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)

    }

    private fun setEvents() {
        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }
    }

    override fun observeViewModel() {

        with(viewModel) {
            itemProposalList.observe(viewLifecycleOwner) { it ->
                listProposal = ArrayList()
                if (it.list.isNotEmpty()) {
                    it.list.forEach {

                        listProposal.add(it)

                    }
                }
                adapter = ItemProposalAdapter(listProposal, object : OnItemProposalClickListener {
                    override fun onItemProposalClickListener(item: ItemProposal) {

                    }
                }, onPasillerosItemClickListener = pasillerosItemClickListener!!)

                binding.locationRecyclerview.adapter = adapter
                layoutManager =
                    LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
                binding.locationRecyclerview.layoutManager = layoutManager
            }

        }

    }
}


