package es.verdnatura.presentation.view.feature.packaging.fragment

import android.content.Context
import android.os.Bundle
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentPackagingBinding
import es.verdnatura.domain.ConstAndValues.ENTRYID
import es.verdnatura.domain.ConstAndValues.ENTRYOBSERVATIONORIGINAL
import es.verdnatura.domain.ConstAndValues.SUPPLIERID
import es.verdnatura.domain.ConstAndValues.SUPPLIERNAME
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.commom.NameWithId
import es.verdnatura.presentation.view.commom.SearchableAdapter
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.packaging.model.EntrySalix
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import kotlinx.coroutines.runBlocking

class SupplierFragment(
    var entryPoint: String = ""

) : BaseFragment<FragmentPackagingBinding, SupplierViewModel>(
    SupplierViewModel::class
) {

    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var layoutManager: LinearLayoutManager? = null
    private var onBack = false
    private val entries = ArrayList<EntrySalix>()
    private lateinit var customDialog: CustomDialog

    companion object {
        fun newInstance(entryPoint: String) = SupplierFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_packaging

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        viewModel.getSuppliers()
        super.onCreate(savedInstanceState)
    }

    override fun onPause() {
        onBack = true
        super.onPause()

    }

    override fun init() {
        binding.filterSupplier.setOnClickListener {
            binding.searchableSuplierRecyclerView.visibility = View.VISIBLE
            binding.searchableSuplierRecyclerView.setSearchHint(getString(R.string.supplierSearch))

        }
        binding.filterEntry.setOnClickListener {
            binding.searchableEntriesRecyclerView.visibility = View.VISIBLE
            binding.searchableEntriesRecyclerView.setSearchHint(getString(R.string.entrySearch))

        }
        customDialog = CustomDialog(requireContext())
        ma.hideBottomNavigation(View.GONE)
        setToolBar()
        setEvents()

        super.init()
    }

    private fun setSearchable(listNames: MutableList<NameWithId>) {

        val adapter =
            SearchableAdapter(
                listElements = listNames,
                context = requireContext()
            ) { elementSelected ->
                binding.filterSupplier.text = (elementSelected.name)
                runBlocking {
                    mobileApplication.dataStoreApp.editDataStoreKey(
                        SUPPLIERNAME,
                        elementSelected.name
                    )
                    mobileApplication.dataStoreApp.editDataStoreKey(
                        SUPPLIERID,
                        elementSelected.id
                    )
                }
                viewModel.getEntriesFromSupplier(elementSelected.id)
                binding.searchableSuplierRecyclerView.visibility = View.GONE

            }

        binding.searchableSuplierRecyclerView.setAdapter(adapter, listNames)

    }

    private fun setSearchableEntries(listNames: MutableList<NameWithId>) {

        val adapter =
            SearchableAdapter(
                listElements = listNames,
                context = requireContext()
            ) { elementSelected ->
                binding.filterEntry.text = elementSelected.name
                runBlocking {
                    mobileApplication.dataStoreApp.editDataStoreKey(
                        ENTRYID,
                        elementSelected.id
                    )
                    mobileApplication.dataStoreApp.editDataStoreKey(
                        ENTRYOBSERVATIONORIGINAL,
                        entries.find { it.id == elementSelected.id }?.observation ?: ""
                    )
                    mobileApplication.dataStoreApp.deleteImages()
                }
                ma.onPasillerosItemClickListener(
                    PasillerosItemVO(title = R.string.titlePackagingCount),
                    getString(R.string.titlePackagingCount)
                )
                binding.searchableEntriesRecyclerView.visibility = View.GONE

            }

        binding.searchableEntriesRecyclerView.setAdapter(adapter, listNames)

    }

    private fun setToolBar() {
        binding.mainToolbar.toolbarTitle.text = entryPoint
    }

    private fun setEvents() {

        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }

    }

    private fun setEntryDialog() {
        binding.filterEntry.visibility = View.VISIBLE

    }

    override fun observeViewModel() {
        with(viewModel) {
            supplierList.observe(viewLifecycleOwner) { item ->
                setSearchable(item.list.map {
                    NameWithId(
                        it.id!!,
                        it.name!!
                    )
                } as MutableList<NameWithId>)
            }
            entryList.observe(viewLifecycleOwner) {
                createEntryList(it.list)
            }


            entry.observe(viewLifecycleOwner) {
                if (it.isError) {
                    ma.messageWithSound(it.errorMessage, isError = true, isPlayed = false)

                } else {
                    runBlocking {
                        mobileApplication.dataStoreApp.editDataStoreKey("ENTRYID", it.id!!.toInt())
                    }

                    ma.onPasillerosItemClickListener(
                        PasillerosItemVO(title = R.string.titlePackagingCount),
                        getString(R.string.titlePackagingCount)
                    )
                }
            }
        }
    }

    private fun createEntryList(list: List<EntrySalix>) {
        list.sortedBy { it.travel?.landed }
        entries.clear()
        if (list.isNotEmpty() && !list[0].isError) {
            setSearchableEntries(list.map { entry ->
                NameWithId(
                    entry.id!!,
                    entry.id.toString() + "->" + (entry.travel?.landed)
                )
            } as MutableList<NameWithId>)
            entries.addAll(list.sortedBy { it.travel?.landed })
        }
        binding.filterEntry.visibility = View.VISIBLE

    }

}

