package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.content.Context
import android.graphics.drawable.Drawable
import android.view.View
import androidx.core.content.res.ResourcesCompat
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionPalletDetailBinding
import es.verdnatura.domain.ConstAndValues.PRINTERNAME
import es.verdnatura.domain.notNull
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPalletClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.paletizador.adapter.ExpeditionPalletViewAdapter
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionTruckVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletViewListVO

class ExpeditionPalletDetailFragment(
    var itemExpeditionTruckVO: ItemExpeditionTruckVO? = null,
    var itemPallet: ItemPalletVO? = null
) : BaseFragment<FragmentExpeditionPalletDetailBinding, ExpeditionPalletDetailViewModel>(
    ExpeditionPalletDetailViewModel::class
) {

    private var adapter: ExpeditionPalletViewAdapter? = null
    private var onPalletClickListener: OnPalletClickListener? = null
    private lateinit var customDialog: CustomDialog

    companion object {
        fun newInstance(item: ItemExpeditionTruckVO, itemPallet: ItemPalletVO) =
            ExpeditionPalletDetailFragment(item, itemPallet)
    }

    override fun onAttach(context: Context) {
        if (context is OnPalletClickListener) onPalletClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.fragment_expedition_pallet_detail

    override fun init() {
        println("ExpeditionPalletDetail")
        customDialog = CustomDialog(requireContext())
        binding.expeditionPalletDetailPallet.text =
            buildString {
                append(getString(R.string.pallet))
                append(" ")
                append(itemPallet!!.Pallet)
            }
        ma.hideBottomNavigation(View.GONE)
        binding.mainToolbar.toolbarTitle.text =
            buildString {
                append(itemExpeditionTruckVO!!.eta)
                append(" ")
                append(
                    itemExpeditionTruckVO!!.description
                )
            }

        setToolBar()
        setEvents()
        viewModel.expeditionPalletView(
            itemPallet!!.Pallet
        )
        super.init()
    }

    private fun setEvents() {
        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }

        binding.buttonOk.setOnClickListener {
            ma.openFragmentTruckFragment()

        }
    }

    private fun setToolBar() {
        val listIcons: ArrayList<Drawable> = ArrayList()
        val iconReload: Drawable =
            ResourcesCompat.getDrawable(resources, R.drawable.ic_autorenew_black_24dp, null)!!
        val iconEditar: Drawable =
            ResourcesCompat.getDrawable(resources, R.drawable.ic_mode_edit_black_24dp, null)!!
        val iconBorrar: Drawable =
            ResourcesCompat.getDrawable(resources, R.drawable.ic_delete_forever_black_24dp, null)!!
        val iconPrint: Drawable =
            ResourcesCompat.getDrawable(resources, R.drawable.ic_print_black_24dp, null)!!

        listIcons.add(iconEditar)
        listIcons.add(iconBorrar)
        listIcons.add(iconPrint)
        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapter(listIcons, object : OnOptionsSelectedListener {
                override fun onOptionsItemSelected(item: Drawable) {
                    if (item == iconReload) {
                        viewModel.expeditionPalletView(
                            itemPallet!!.Pallet
                        )
                    } else if (item == iconEditar) {
                        onPalletClickListener!!.onPalletClickListener(
                            itemExpeditionTruckVO!!,
                            itemPallet!!
                        )
                    } else if (item == iconBorrar) {

                        customDialog.setTitle(getString(R.string.info))
                        customDialog.setDescription(getString(R.string.messageConfirm))
                        customDialog.setOkButton(getString(R.string.delete)) {
                            viewModel.expeditionPalletDel(
                                itemPallet!!.Pallet
                            )
                            customDialog.dismiss()
                        }
                        customDialog.setKoButton(getString(R.string.cancel)) {
                            customDialog.dismiss()
                        }
                        customDialog.show()

                    } else if (item == iconPrint) {

                        if (ma.havePrinter() && ma.haveSector()) {
                            (getString(R.string.Imprimiendo) + mobileApplication.dataStoreApp.readDataStoreKey<String>(
                                PRINTERNAME
                            )).toast(
                                requireContext()
                            )
                            viewModel.expeditionPalletPrintSet(
                                itemPallet!!.Pallet
                            )
                        } else {
                            ma.messageWithSound(
                                getString(R.string.printerFault),
                                isError = true,
                                true,
                                getString(R.string.printError),
                                false
                            )
                        }
                    }
                }
            })
        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    override fun observeViewModel() {
        with(viewModel) {
            loadExpeditionPalletList.observe(viewLifecycleOwner) { event ->
                event.getContentIfNotHandled().notNull { printExpeditionList(it) }
            }

            response.observe(viewLifecycleOwner) {
                ma.openFragmentTruckFragment()
            }

            responsePrintPallet.observe(viewLifecycleOwner) {
                viewModel.expeditionStateAddByPallet(
                    itemPallet!!.Pallet,
                    "PALLETIZED"
                )
            }
        }
    }

    private fun printExpeditionList(it: ItemPalletViewListVO) {
        val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        adapter = ExpeditionPalletViewAdapter(it.list)
        binding.expeditionPalletRecyclerview.adapter = adapter
        binding.expeditionPalletRecyclerview.layoutManager = lm
    }

}