package es.verdnatura.presentation.view.feature.qr

import android.view.inputmethod.EditorInfo
import es.verdnatura.R
import es.verdnatura.databinding.FragmentQrBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.view.component.CustomDialog

@Suppress("UNUSED_ANONYMOUS_PARAMETER")
class QrFragment(
    var entryPoint: String = ""
) : BaseFragment<FragmentQrBinding, QrFragmentViewModel>(QrFragmentViewModel::class) {

    private var goBack: Boolean = false
    override fun getLayoutId(): Int = R.layout.fragment_qr
    private lateinit var customDialog: CustomDialog
    private var itemscaned = ""

    companion object {
        fun newInstance(entryPoint: String) = QrFragment(entryPoint)
    }

    override fun init() {

        customDialog = CustomDialog(requireContext())
        setToolbar()
        setEvents()
        super.init()
    }

    private fun setToolbar() {

        binding.mainToolbar.toolbarTitle.text = entryPoint
    }

    override fun onPause() {
        goBack = true
        super.onPause()
    }

    private fun setEvents() {

        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }

        binding.scanInput.requestFocus()
        binding.scanInput.setOnEditorActionListener { v, actionId, event ->

            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {

                goBack = false
                if (!binding.scanInput.text.isNullOrEmpty()) {
                    /*  viewModel.qr_getCall(
                          binding.scanInput.text.toString()
                      )*/
                    itemscaned = binding.scanInput.text.toString()
                }

            }
            binding.scanInput.setText("")
            ma.hideKeyboard(binding.scanInput)
            return@setOnEditorActionListener true
        }

    }

    override fun observeViewModel() {

        with(viewModel) {
            qrresponse.observe(viewLifecycleOwner) {

                if (it.isError) {

                    ma.messageWithSound(it.errorMessage + " " + itemscaned, true, false)

                } else {

                    customDialog.setTitle(getString(R.string.info))
                        .setDescription(
                            getString(R.string.operation) + itemscaned + "." + getString(
                                R.string.answer
                            ) + it.response
                        )
                        .setOkButton(getString(R.string.accept)) {
                            customDialog.dismiss()
                            binding.mainToolbar.backButton.performClick()
                        }.show()

                }

            }
        }

    }

}

